import java.util.Date;
import java.util.List;
import java.util.UUID;

import javax.persistence.Basic;
import javax.persistence.CascadeType;
import javax.persistence.ElementCollection;
import javax.persistence.Embeddable;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FlushModeType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;

public class QuerysSlowerWhenPersistManyEntities {
	
	private static long time;

	public static void main(String[] args) {
		System.out.println("start");
		
		test(1000);
		test(10000);
	}
	
	public static void test(int numberOfEntities) {
		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("objectdb:$objectdb/db/test.tmp;drop");
		EntityManager em = emf.createEntityManager();
		em.setFlushMode(FlushModeType.AUTO);

		// Persist an entity with an embeddable:
		remarksTime();
		em.getTransaction().begin();
		for (int i = 0; i < numberOfEntities; i++) {
			
            Message msg = new MessageImpl();
            msg.setTypeID(UUID.randomUUID().toString()
                    + "1234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890");
			em.persist(msg);

			Query query = em.createQuery("select m from MessageImpl m where uid = ?1");
			query.setParameter(1, msg.getUid());
			List<MessageImpl> result = query.getResultList();
		}
		em.getTransaction().commit();
		printTimeDiff();
		
		em.close();
		emf.close();
	}
	
	private static void remarksTime() {
		time = System.currentTimeMillis();
	}

	private static void printTimeDiff() {
		System.out.println(System.currentTimeMillis() - time);
	}

	@Entity
	public static class MessageImpl extends ModelElementImpl implements Message {

	    @ManyToOne (cascade = { CascadeType.REFRESH },
	        targetEntity = ModelElementImpl.class)
	    private ModelElement causingObject;

	    @Basic
	    private MessageSeverity severity;

	    @Basic
	    private String typeID;

	    @ElementCollection (targetClass = TokenImpl.class)
	    private List<Token> messageTokens;

	    @ElementCollection (targetClass = TokenImpl.class)
	    private List<Token> hintTokens;

	    @Basic
	    @Temporal (TemporalType.TIMESTAMP)
	    private Date date;

	    /**
	     * Construct a new message and set its date to this time of instantiation.
	     */
	    public MessageImpl() {
	        super();
	        this.date = new Date();
	    }

	    @Override
	    public MessageSeverity getSeverity() {
	        return severity;
	    }

	    @Override
	    public void setSeverity(MessageSeverity severity) {
	        this.severity = severity;
	    }

	    @Override
	    public void setTypeID(String typeID) {
	        this.typeID = typeID;
	    }

	    @Override
	    public void setCausingObject(ModelElement causingObject) {
	        if (causingObject != null) {
	            this.causingObject = causingObject;
	        }
	    }

	    @Override
	    public String getTypeID() {
	        return this.typeID;
	    }

	    @Override
	    public ModelElement getCausingObject() {
	        return causingObject;
	    }

	    @Override
	    public void setMessage(List<Token> messageTokens) {
	        this.messageTokens = messageTokens;
	    }

	    @Override
	    public void setHint(List<Token> hintTokens) {
	        this.hintTokens = hintTokens;
	    }

	    @Override
	    public List<Token> getMessage() {
	        return messageTokens;
	    }

	    @Override
	    public List<Token> getHint() {
	        return hintTokens;
	    }

	    @Override
	    public void addMessageToken(Token t) {
	        messageTokens.add(t);
	    }

	    @Override
	    public void addHintToken(Token t) {
	        hintTokens.add(t);
	    }

	    @Override
	    public String toString() {
	        StringBuffer buf = new StringBuffer();
	        buf.append("[message: type: ");
	        buf.append(this.getTypeID());
	        buf.append(", ");
	        for (Message.Token t : getMessage()) {
	            buf.append(t.toString());
	        }
	        buf.append(", hint: ");
	        if (getHint() != null) {
	            for (Message.Token t : getHint()) {
	                buf.append(t.toString());
	            }
	        }
	        buf.append("]");

	        return buf.toString();
	    }

	    @Override
	    public Date getDate() {
	        return this.date;
	    }

	    public void setDate(Date created) {
	        this.date = created;
	    }
	}

	@Entity
	public static abstract class ModelElementImpl implements ModelElement {

	    @Id
	    private String uid;

	    protected ModelElementImpl() {
	        this.uid = UUID.randomUUID().toString().toUpperCase().replaceAll("-", "");
	    }

	    @Override
	    public int hashCode() {
	        if (uid == null) {
	            return super.hashCode();
	        }

	        return uid.hashCode();
	    }

	    @Override
	    public boolean equals(Object obj) {
	        if (obj instanceof ModelElement && this.getUid() != null) {
	            return this.getUid().equals(((ModelElement)obj).getUid());
	        }

	        return super.equals(obj);
	    }

	    @Override
	    public String getURI() {
	        throw new UnsupportedOperationException();
	    }

	    @Override
	    public final String getUid() {
	        return this.uid;
	    }
	}

	public static interface Message extends ModelElement {

	    public Date getDate();

	    public MessageSeverity getSeverity();

	    public static interface Token {

	        public String toString();
	    }

	    public static interface LinkToken extends Token {

	        public ModelElement getLinkedObject();
	    }

	    public String getTypeID();

	    public List<Token> getMessage();

	    public void setMessage(List<Token> messageTokens);

	    public List<Token> getHint();

	    public void setHint(List<Token> hintTokens);

	    public ModelElement getCausingObject();

	    public void setSeverity(MessageSeverity severity);

	    public void setTypeID(String typeID);

	    public void setCausingObject(ModelElement causingObject);

	    public void addMessageToken(Message.Token t);

	    public void addHintToken(Message.Token t);

	}

	public static abstract interface ModelElement {

	    public String getURI();

	    public String getUid();

	    @Override
	    public boolean equals(Object obj);

	    @Override
	    public int hashCode();
	}

	public static enum MessageSeverity {
	    INFO, WARNING, ERROR
	}

	@Embeddable
	public static class TokenImpl implements Token {

	    @Basic
	    String string;

	    protected TokenImpl() {
	    }

	    public TokenImpl(String string) {
	        this.string = string;
	    }

	    @Override
	    public String toString() {
	        return string;
	    }
	}

    public static interface Token {

        public String toString();
    }
}
